@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.198.2.19 2006/01/03 22:16:14 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.198.2.21 2007/01/02 21:43:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2006 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -185,6 +185,71 @@ struct option_helper {
 	{ BER_BVNULL, 0, NULL, NULL }
 };
 
+int
+parse_debug_unknowns( char **unknowns, int *levelp )
+{
+	int i, level, rc = 0;
+
+	for ( i = 0; unknowns[ i ] != NULL; i++ ) {
+		level = 0;
+		if ( str2loglevel( unknowns[ i ], &level )) {
+			fprintf( stderr,
+				"unrecognized log level \"%s\"\n", unknowns[ i ] );
+			rc = 1;
+		} else {
+			*levelp |= level;
+		}
+	}
+	return rc;
+}
+
+int
+parse_debug_level( const char *arg, int *levelp, char ***unknowns )
+{
+	int	level;
+
+	if ( arg != NULL && arg[ 0 ] != '-' && !isdigit( arg[ 0 ] ) )
+	{
+		int	i;
+		char	**levels;
+
+		levels = ldap_str2charray( arg, "," );
+
+		for ( i = 0; levels[ i ] != NULL; i++ ) {
+			level = 0;
+
+			if ( str2loglevel( levels[ i ], &level ) ) {
+				/* remember this for later */
+				ldap_charray_add( unknowns, levels[ i ] );
+				fprintf( stderr,
+					"unrecognized log level \"%s\" (deferred)\n",
+					levels[ i ] );
+			} else {
+				*levelp |= level;
+			}
+		}
+
+		ldap_charray_free( levels );
+
+	} else {
+		if ( lutil_atoix( &level, arg, 0 ) != 0 ) {
+			fprintf( stderr,
+				"unrecognized log level "
+				"\"%s\"\n", arg );
+			return 1;
+		}
+
+		if ( level == 0 ) {
+			*levelp = 0;
+
+		} else {
+			*levelp |= level;
+		}
+	}
+
+	return 0;
+}
+
 static void
 usage( char *name )
 {
@@ -261,6 +326,9 @@ int main( int argc, char **argv )
 	struct sync_cookie *scp = NULL;
 	struct sync_cookie *scp_entry = NULL;
 
+	char **debug_unknowns = NULL;
+	char **syslog_unknowns = NULL;
+
 	char *serverNamePrefix = "";
 	size_t	l;
 
@@ -395,46 +463,13 @@ int main( int argc, char **argv )
 			int	level = 0;
 
 			no_detach = 1;
-#ifdef LDAP_DEBUG
-			if ( optarg != NULL && optarg[ 0 ] != '-' && !isdigit( optarg[ 0 ] ) )
-			{
-				int	i, goterr = 0;
-				char	**levels;
-
-				levels = ldap_str2charray( optarg, "," );
-
-				for ( i = 0; levels[ i ] != NULL; i++ ) {
-					level = 0;
-
-					if ( str2loglevel( levels[ i ], &level ) ) {
-						fprintf( stderr,
-							"unrecognized log level "
-							"\"%s\"\n", levels[ i ] );
-						goterr = 1;
-						/* but keep parsing... */
-
-					} else {
-						slap_debug |= level;
-					}
-				}
-
-				ldap_charray_free( levels );
-
-				if ( goterr ) {
-					goto destroy;
-				}
-
-			} else {
-				if ( lutil_atoix( &level, optarg, 0 ) != 0 ) {
-					fprintf( stderr,
-						"unrecognized log level "
-						"\"%s\"\n", optarg );
-					goto destroy;
-				}
-				slap_debug |= level;
+			if ( parse_debug_level( optarg, &level, &debug_unknowns ) ) {
+				goto destroy;
 			}
+#ifdef LDAP_DEBUG
+			slap_debug |= level;
 #else
-			if ( lutil_atoi( &level, optarg ) != 0 || level != 0 )
+			if ( level != 0 )
 				fputs( "must compile with LDAP_DEBUG for debugging\n",
 				       stderr );
 #endif
@@ -483,8 +518,7 @@ int main( int argc, char **argv )
 		}
 
 		case 's':	/* set syslog level */
-			if ( lutil_atoi( &ldap_syslog, optarg ) != 0 ) {
-				fprintf( stderr, "unable to parse syslog level \"%s\"", optarg );
+			if ( parse_debug_level( optarg, &ldap_syslog, &syslog_unknowns ) ) {
 				goto destroy;
 			}
 			break;
@@ -641,6 +675,21 @@ unhandled_option:;
 		goto destroy;
 	}
 
+	if ( debug_unknowns ) {
+		rc = parse_debug_unknowns( debug_unknowns, &slap_debug );
+		ldap_charray_free( debug_unknowns );
+		debug_unknowns = NULL;
+		if ( rc )
+			goto destroy;
+	}
+	if ( syslog_unknowns ) {
+		rc = parse_debug_unknowns( syslog_unknowns, &ldap_syslog );
+		ldap_charray_free( syslog_unknowns );
+		syslog_unknowns = NULL;
+		if ( rc )
+			goto destroy;
+	}
+
 	if ( check & CHECK_CONFIG ) {
 		fprintf( stderr, "config check succeeded\n" );
 
